package nl.amis.spring.jdbc;
import java.sql.ResultSet;
import java.sql.SQLException;

import java.util.Iterator;
import java.util.List;

import javax.sql.DataSource;

import nl.amis.spring.hrm.EmpDAO;
import nl.amis.spring.hrm.Employee;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.RowMapperResultReader;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

public class EmployeeJdbcDAO extends JdbcDaoSupport implements EmpDAO  {

   public long getSalarySum() {
        JdbcTemplate jt = getJdbcTemplate();
        return jt.queryForLong ( "select sum(sal) from emp");
    }//getSalarySum    
     
    public List getAllEmployees() {
        JdbcTemplate jt = getJdbcTemplate();
        return jt.query ( "select ename, job from emp"
                        , new RowMapperResultReader(new EmployeeRowMapper())
                        );
    }//getAllEmployees    
        
    // this inner class is invoked for each Row om the ResultSet. It implements
    // the Spring RowMapper interface, that is used to convert ResultSet records
    // to Domain Objects. RowMappers are used when the results are not simple,
    // like in the getSalarySum example above
    class EmployeeRowMapper implements RowMapper {
      public Object mapRow(ResultSet rs, int index) throws SQLException {
        Employee emp = new Employee();
        emp.setLastName(rs.getString(1));
        emp.setJobTitle(rs.getString(2));
        return emp;
      }      
    }//class EmployeeRowMapper


public static void main(String[] args)  throws Exception {
    
    ApplicationContext ctx = new ClassPathXmlApplicationContext("SpringConfig.xml");
    DataSource ds = (DataSource)ctx.getBean("dataSourceDBDirect");
    EmployeeJdbcDAO empDao = new EmployeeJdbcDAO();
    // invoke the setDataSource() method on the superclass
    empDao.setDataSource(ds);

    Iterator empIter = empDao.getAllEmployees().iterator();
    while (empIter.hasNext()) {
      Employee emp = (Employee)empIter.next();
      System.out.println(emp.getLastName()+" ("+emp.getJobTitle()+")");
    }//while
    System.out.println("The Salary Sum = "+empDao.getSalarySum());
}// main

}// EmployeeJdbcDao